@@ -15,23 +15,29 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 """Operators that integrates with Google Cloud Build service."""
+from __future__ import annotations
 
 import json
 import re
-import warnings
 from copy import deepcopy
-from typing import TYPE_CHECKING, Any, Dict, Optional, Sequence, Tuple, Union
-from urllib.parse import unquote, urlparse
+from typing import TYPE_CHECKING, Any, Sequence
+from urllib.parse import unquote, urlsplit
 
-import yaml
+from google.api_core.gapic_v1.method import DEFAULT, _MethodDefault
 from google.api_core.retry import Retry
 from google.cloud.devtools.cloudbuild_v1.types import Build, BuildTrigger, RepoSource
 
 from airflow.exceptions import AirflowException
 from airflow.models import BaseOperator
 from airflow.providers.google.cloud.hooks.cloud_build import CloudBuildHook
+from airflow.providers.google.cloud.links.cloud_build import (
+    CloudBuildLink,
+    CloudBuildListLink,
+    CloudBuildTriggerDetailsLink,
+    CloudBuildTriggersListLink,
+)
+from airflow.utils import yaml
 
 if TYPE_CHECKING:
     from airflow.utils.context import Context
@@ -66,21 +72,21 @@ class CloudBuildCancelBuildOperator(BaseOperator):
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
 
-    :rtype: dict
     """
 
     template_fields: Sequence[str] = ("project_id", "id_", "gcp_conn_id")
+    operator_extra_links = (CloudBuildLink(),)
 
     def __init__(
         self,
         *,
         id_: str,
-        project_id: Optional[str] = None,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        project_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -92,7 +98,7 @@ class CloudBuildCancelBuildOperator(BaseOperator):
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
         result = hook.cancel_build(
             id_=self.id_,
@@ -101,6 +107,16 @@ class CloudBuildCancelBuildOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+
+        self.xcom_push(context, key="id", value=result.id)
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+                build_id=result.id,
+            )
         return Build.to_dict(result)
 
 
@@ -112,11 +128,8 @@ class CloudBuildCreateBuildOperator(BaseOperator):
         For more information on how to use this operator, take a look at the guide:
         :ref:`howto/operator:CloudBuildCreateBuildOperator`
 
-    :param build: Optional, the build resource to create. If a dict is provided, it must be of
+    :param build: The build resource to create. If a dict is provided, it must be of
         the same form as the protobuf message `google.cloud.devtools.cloudbuild_v1.types.Build`.
-        Only either build or body should be passed.
-    :param body: (Deprecated) The build resource to create.
-        This parameter has been deprecated. You should pass the build parameter instead.
     :param project_id: Optional, Google Cloud Project project_id where the function belongs.
         If set to None or missing, the default project_id from the GCP connection is used.
     :param wait: Optional, wait for operation to finish.
@@ -135,23 +148,22 @@ class CloudBuildCreateBuildOperator(BaseOperator):
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
 
-    :rtype: dict
     """
 
-    template_fields: Sequence[str] = ("project_id", "build", "body", "gcp_conn_id", "impersonation_chain")
+    template_fields: Sequence[str] = ("project_id", "build", "gcp_conn_id", "impersonation_chain")
+    operator_extra_links = (CloudBuildLink(),)
 
     def __init__(
         self,
         *,
-        build: Optional[Union[Dict, Build]] = None,
-        body: Optional[Dict] = None,
-        project_id: Optional[str] = None,
+        build: dict | Build,
+        project_id: str | None = None,
         wait: bool = True,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -162,37 +174,21 @@ class CloudBuildCreateBuildOperator(BaseOperator):
         self.metadata = metadata
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
-        self.body = body
-
-        if body and build:
-            raise AirflowException("You should not pass both build or body parameters. Both are set.")
-        if body is not None:
-            warnings.warn(
-                "The body parameter has been deprecated. You should pass body using the build parameter.",
-                DeprecationWarning,
-                stacklevel=4,
-            )
-            actual_build = body
-        else:
-            if build is None:
-                raise AirflowException("You should pass one of the build or body parameters. Both are None")
-            actual_build = build
-
-        self.build = actual_build
+        self.build = build
         # Not template fields to keep original value
-        self.build_raw = actual_build
+        self.build_raw = build
 
     def prepare_template(self) -> None:
         # if no file is specified, skip
         if not isinstance(self.build_raw, str):
             return
         with open(self.build_raw) as file:
-            if any(self.build_raw.endswith(ext) for ext in ['.yaml', '.yml']):
+            if any(self.build_raw.endswith(ext) for ext in [".yaml", ".yml"]):
                 self.build = yaml.safe_load(file.read())
-            if self.build_raw.endswith('.json'):
+            if self.build_raw.endswith(".json"):
                 self.build = json.loads(file.read())
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
 
         build = BuildProcessor(build=self.build).process_body()
@@ -205,6 +201,16 @@ class CloudBuildCreateBuildOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+
+        self.xcom_push(context, key="id", value=result.id)
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+                build_id=result.id,
+            )
         return Build.to_dict(result)
 
 
@@ -235,21 +241,24 @@ class CloudBuildCreateBuildTriggerOperator(BaseOperator):
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
 
-    :rtype: dict
     """
 
     template_fields: Sequence[str] = ("project_id", "trigger", "gcp_conn_id")
+    operator_extra_links = (
+        CloudBuildTriggersListLink(),
+        CloudBuildTriggerDetailsLink(),
+    )
 
     def __init__(
         self,
         *,
-        trigger: Union[dict, BuildTrigger],
-        project_id: Optional[str] = None,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        trigger: dict | BuildTrigger,
+        project_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -261,7 +270,7 @@ class CloudBuildCreateBuildTriggerOperator(BaseOperator):
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
         result = hook.create_build_trigger(
             trigger=self.trigger,
@@ -270,6 +279,20 @@ class CloudBuildCreateBuildTriggerOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+        self.xcom_push(context, key="id", value=result.id)
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildTriggerDetailsLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+                trigger_id=result.id,
+            )
+            CloudBuildTriggersListLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+            )
         return BuildTrigger.to_dict(result)
 
 
@@ -301,17 +324,18 @@ class CloudBuildDeleteBuildTriggerOperator(BaseOperator):
     """
 
     template_fields: Sequence[str] = ("project_id", "trigger_id", "gcp_conn_id")
+    operator_extra_links = (CloudBuildTriggersListLink(),)
 
     def __init__(
         self,
         *,
         trigger_id: str,
-        project_id: Optional[str] = None,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        project_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -323,7 +347,7 @@ class CloudBuildDeleteBuildTriggerOperator(BaseOperator):
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
         hook.delete_build_trigger(
             trigger_id=self.trigger_id,
@@ -332,6 +356,13 @@ class CloudBuildDeleteBuildTriggerOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildTriggersListLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+            )
 
 
 class CloudBuildGetBuildOperator(BaseOperator):
@@ -360,21 +391,21 @@ class CloudBuildGetBuildOperator(BaseOperator):
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
 
-    :rtype: dict
     """
 
     template_fields: Sequence[str] = ("project_id", "id_", "gcp_conn_id")
+    operator_extra_links = (CloudBuildLink(),)
 
     def __init__(
         self,
         *,
         id_: str,
-        project_id: Optional[str] = None,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        project_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -386,7 +417,7 @@ class CloudBuildGetBuildOperator(BaseOperator):
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
         result = hook.get_build(
             id_=self.id_,
@@ -395,6 +426,14 @@ class CloudBuildGetBuildOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+                build_id=result.id,
+            )
         return Build.to_dict(result)
 
 
@@ -424,21 +463,21 @@ class CloudBuildGetBuildTriggerOperator(BaseOperator):
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
 
-    :rtype: dict
     """
 
     template_fields: Sequence[str] = ("project_id", "trigger_id", "gcp_conn_id")
+    operator_extra_links = (CloudBuildTriggerDetailsLink(),)
 
     def __init__(
         self,
         *,
         trigger_id: str,
-        project_id: Optional[str] = None,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        project_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -450,7 +489,7 @@ class CloudBuildGetBuildTriggerOperator(BaseOperator):
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
         result = hook.get_build_trigger(
             trigger_id=self.trigger_id,
@@ -459,6 +498,14 @@ class CloudBuildGetBuildTriggerOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildTriggerDetailsLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+                trigger_id=result.id,
+            )
         return BuildTrigger.to_dict(result)
 
 
@@ -490,23 +537,23 @@ class CloudBuildListBuildTriggersOperator(BaseOperator):
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
 
-    :rtype: List[dict]
     """
 
     template_fields: Sequence[str] = ("location", "project_id", "gcp_conn_id")
+    operator_extra_links = (CloudBuildTriggersListLink(),)
 
     def __init__(
         self,
         *,
         location: str,
-        project_id: Optional[str] = None,
-        page_size: Optional[int] = None,
-        page_token: Optional[str] = None,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        project_id: str | None = None,
+        page_size: int | None = None,
+        page_token: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -520,7 +567,7 @@ class CloudBuildListBuildTriggersOperator(BaseOperator):
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
         results = hook.list_build_triggers(
             project_id=self.project_id,
@@ -531,6 +578,13 @@ class CloudBuildListBuildTriggersOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildTriggersListLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+            )
         return [BuildTrigger.to_dict(result) for result in results]
 
 
@@ -562,23 +616,23 @@ class CloudBuildListBuildsOperator(BaseOperator):
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
 
-    :rtype: List[dict]
     """
 
     template_fields: Sequence[str] = ("location", "project_id", "gcp_conn_id")
+    operator_extra_links = (CloudBuildListLink(),)
 
     def __init__(
         self,
         *,
         location: str,
-        project_id: Optional[str] = None,
-        page_size: Optional[int] = None,
-        filter_: Optional[str] = None,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        project_id: str | None = None,
+        page_size: int | None = None,
+        filter_: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -592,7 +646,7 @@ class CloudBuildListBuildsOperator(BaseOperator):
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
         results = hook.list_builds(
             project_id=self.project_id,
@@ -603,6 +657,9 @@ class CloudBuildListBuildsOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildListLink.persist(context=context, task_instance=self, project_id=project_id)
         return [Build.to_dict(result) for result in results]
 
 
@@ -634,22 +691,22 @@ class CloudBuildRetryBuildOperator(BaseOperator):
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
 
-    :rtype: dict
     """
 
     template_fields: Sequence[str] = ("project_id", "id_", "gcp_conn_id")
+    operator_extra_links = (CloudBuildLink(),)
 
     def __init__(
         self,
         *,
         id_: str,
-        project_id: Optional[str] = None,
+        project_id: str | None = None,
         wait: bool = True,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -662,7 +719,7 @@ class CloudBuildRetryBuildOperator(BaseOperator):
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
         result = hook.retry_build(
             id_=self.id_,
@@ -672,6 +729,16 @@ class CloudBuildRetryBuildOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+
+        self.xcom_push(context, key="id", value=result.id)
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+                build_id=result.id,
+            )
         return Build.to_dict(result)
 
 
@@ -704,23 +771,23 @@ class CloudBuildRunBuildTriggerOperator(BaseOperator):
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
 
-    :rtype: dict
     """
 
     template_fields: Sequence[str] = ("project_id", "trigger_id", "source", "gcp_conn_id")
+    operator_extra_links = (CloudBuildLink(),)
 
     def __init__(
         self,
         *,
         trigger_id: str,
-        source: Union[dict, RepoSource],
-        project_id: Optional[str] = None,
+        source: dict | RepoSource,
+        project_id: str | None = None,
         wait: bool = True,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -734,7 +801,7 @@ class CloudBuildRunBuildTriggerOperator(BaseOperator):
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
         result = hook.run_build_trigger(
             trigger_id=self.trigger_id,
@@ -745,6 +812,15 @@ class CloudBuildRunBuildTriggerOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+        self.xcom_push(context, key="id", value=result.id)
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+                build_id=result.id,
+            )
         return Build.to_dict(result)
 
 
@@ -776,22 +852,22 @@ class CloudBuildUpdateBuildTriggerOperator(BaseOperator):
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
 
-    :rtype: dict
     """
 
     template_fields: Sequence[str] = ("project_id", "trigger_id", "trigger", "gcp_conn_id")
+    operator_extra_links = (CloudBuildTriggerDetailsLink(),)
 
     def __init__(
         self,
         *,
         trigger_id: str,
-        trigger: Union[dict, BuildTrigger],
-        project_id: Optional[str] = None,
-        retry: Optional[Retry] = None,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        trigger: dict | BuildTrigger,
+        project_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -804,7 +880,7 @@ class CloudBuildUpdateBuildTriggerOperator(BaseOperator):
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudBuildHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
         result = hook.update_build_trigger(
             trigger_id=self.trigger_id,
@@ -814,6 +890,15 @@ class CloudBuildUpdateBuildTriggerOperator(BaseOperator):
             timeout=self.timeout,
             metadata=self.metadata,
         )
+        self.xcom_push(context, key="id", value=result.id)
+        project_id = self.project_id or hook.project_id
+        if project_id:
+            CloudBuildTriggerDetailsLink.persist(
+                context=context,
+                task_instance=self,
+                project_id=project_id,
+                trigger_id=result.id,
+            )
         return BuildTrigger.to_dict(result)
 
 
@@ -828,7 +913,7 @@ class BuildProcessor:
         See: https://cloud.google.com/cloud-build/docs/api/reference/rest/Shared.Types/Build
     """
 
-    def __init__(self, build: Union[Dict, Build]) -> None:
+    def __init__(self, build: dict | Build) -> None:
         self.build = deepcopy(build)
 
     def _verify_source(self) -> None:
@@ -869,15 +954,14 @@ class BuildProcessor:
         Processes the body passed in the constructor
 
         :return: the body.
-        :rtype: `google.cloud.devtools.cloudbuild_v1.types.Build`
         """
-        if 'source' in self.build:
+        if "source" in self.build:
             self._verify_source()
             self._reformat_source()
         return Build(self.build)
 
     @staticmethod
-    def _convert_repo_url_to_dict(source: str) -> Dict[str, Any]:
+    def _convert_repo_url_to_dict(source: str) -> dict[str, Any]:
         """
         Convert url to repository in Google Cloud Source to a format supported by the API
 
@@ -888,7 +972,7 @@ class BuildProcessor:
             https://source.cloud.google.com/airflow-project/airflow-repo/+/branch-name:
 
         """
-        url_parts = urlparse(source)
+        url_parts = urlsplit(source)
 
         match = REGEX_REPO_PATH.search(url_parts.path)
 
@@ -911,7 +995,7 @@ class BuildProcessor:
         return source_dict
 
     @staticmethod
-    def _convert_storage_url_to_dict(storage_url: str) -> Dict[str, Any]:
+    def _convert_storage_url_to_dict(storage_url: str) -> dict[str, Any]:
         """
         Convert url to object in Google Cloud Storage to a format supported by the API
 
@@ -922,7 +1006,7 @@ class BuildProcessor:
             gs://bucket-name/object-name.tar.gz
 
         """
-        url_parts = urlparse(storage_url)
+        url_parts = urlsplit(storage_url)
 
         if url_parts.scheme != "gs" or not url_parts.hostname or not url_parts.path or url_parts.path == "/":
             raise AirflowException(
@@ -930,7 +1014,7 @@ class BuildProcessor:
                 "gs://bucket-name/object-name.tar.gz#24565443"
             )
 
-        source_dict: Dict[str, Any] = {
+        source_dict: dict[str, Any] = {
             "bucket": url_parts.hostname,
             "object_": url_parts.path[1:],
         }
